/********************************************************************/
/*  Stringy.cpp  -  Demonstrate the use of the simple Stringy       */
/*                  class modeled on Savitch's StringVar class      */
/*                  Programming Fundamentals I / C++                */
/*  Author:         Bary W Pollack                                  */
/*  Last Update:    02-13-2008                                      */
/********************************************************************/

#pragma warning(disable : 4996)             // for Visual Studio 2008 Express

#include <iostream>
#include <string>

using namespace std;

/********************************************************************/

const int nDefaultLth = 100;                // default maximum string length

void Error(const string sMessage);          // issue error message, then exit

/********************************************************************/

class Stringy {                             // similar to Savitch's StringVar class

  public:
    Stringy(void);                          // default constructor
    Stringy(const int nLength);             // max length nLength constructor
    Stringy(const char sString[]);          // initialized to sString constructor
    Stringy(const Stringy& syString);       // copy constructor
    ~Stringy(void);                         // destructor
    int GetnMaxLth(void) const;             // "get" accessor for nMaxLth
    int GetLength(void) const;              // the true current length
    void operator =(const Stringy& syRString);  // assignment operator

    friend istream& operator >>(istream& ins, Stringy& syString);
    friend ostream& operator <<(ostream& outs, const Stringy& syString);

  private:
    int  nMaxLth;                           // maximum data length
    char *sData;                            // the string data(as a C-string)
};

/********************************************************************/

Stringy::Stringy(void) {                    // default constructor
    if ((sData = new char[nDefaultLth + 1]) == NULL)
        Error("Insufficient memory 0");
    nMaxLth = nDefaultLth;
    *sData = '\0';
}

/********************************************************************/

Stringy::Stringy(const int nLength) {       // constructor, max lth nLength
    if ((sData = new char[nLength + 1]) == NULL)
        Error("Insufficient memory 1");
    nMaxLth = nLength;
    *sData = '\0';
}

/********************************************************************/

Stringy::Stringy(const char sString[]) {    // initialized constructor
    nMaxLth = strlen(sString);
    if ((sData = new char[nMaxLth + 1]) == NULL)
        Error("Insufficient memory 2");
    strcpy(sData, sString);
}

/********************************************************************/

Stringy::Stringy(const Stringy& syString) { // copy constructor
    nMaxLth = syString.GetLength();
    if ((sData = new char[nMaxLth + 1]) == NULL)
        Error("Insufficent memory 3");
    strcpy(sData, syString.sData);
}

/********************************************************************/

Stringy::~Stringy(void) {                   // destructor
    delete [] sData;
    sData = NULL;
    nMaxLth = -1;
}

/********************************************************************/

int Stringy::GetnMaxLth(void) const {       // nMaxLth accessor function
    return nMaxLth;
}

/********************************************************************/

int Stringy::GetLength(void) const {        // current string length
    return strlen(sData);
}

/********************************************************************/

void Stringy::operator =(const Stringy& syRString) {
    if (this != &syRString) {
        int nNewLth = syRString.GetLength();
        if (nNewLth > nMaxLth) {            // guarantee that LHS != RHS
            delete [] sData;
            if ((sData = new char[nMaxLth + 1]) == NULL)
                Error("Insufficient memory 6");
        }
        strcpy(sData, syRString.sData);
    }
}

/********************************************************************/

istream& operator >>(istream& ins, Stringy& syString) {
    char sBuffer[10 * nDefaultLth + 1];		// never enough!!
    ins >> sBuffer;                         // dangerous!!
    int  nLth = strlen(sBuffer);

    if (nLth > syString.GetnMaxLth()) {
        delete [] syString.sData;
        if ((syString.sData = new char[nLth + 1]) == NULL)
            Error("Insufficient memory 4");
    }
    strcpy(syString.sData, sBuffer);
    return ins;
}

/********************************************************************/

ostream& operator <<(ostream& outs, const Stringy& syString) {
    outs << syString.sData;
    return outs;
}

/********************************************************************/

void Error(string sMessage) {               // issue error message; then exit
    cerr << endl << "Error: " << sMessage << endl << endl;
    exit(1);
}

/********************************************************************/

int main(void) {                            // driver
    cout << endl << "Demonstrate use of the Stringy class" << endl << endl;

    Stringy s1, s2(10), s3("Test");
    Stringy s4(s3), s5;

    cout << "s1:  " << s1.GetLength() << " : " << s1 << endl;
    cout << "s2:  " << s2.GetLength() << " : " << s2 << endl;
    cout << "s3:  " << s3.GetLength() << " : " << s3 << endl;
    cout << "s4:  " << s4.GetLength() << " : " << s4 << endl;

    cin >> s2;
    cout << "The input was:(" << s2.GetLength() << ") : " << s2 << endl;

    s5 = s2;
    cout << "Now, s5 contains: " << s5.GetLength() << " : " << s5 << endl;

    cout << endl << "Fini" << endl << endl;
    return 0;
}

/********************************************************************/
/*  Execution output...

Demonstrate use of the Stringy class

s1:  0 :
s2:  0 :
s3:  4 : Test
s4:  4 : Test

Testing_Stringy
The input was:(15) : Testing_Stringy
Now, s5 contains: 15 : Testing_Stringy

Fini

/********************************************************************/

